package com.qmth.exam.reserve.controller.admin;

import java.io.IOException;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.exam.reserve.bean.PagerReq;
import com.qmth.exam.reserve.bean.studentimport.StudentImportTaskVO;
import com.qmth.exam.reserve.controller.BaseController;
import com.qmth.exam.reserve.entity.StudentImportTaskEntity;
import com.qmth.exam.reserve.enums.ImportStatus;
import com.qmth.exam.reserve.service.StudentImportTaskService;
import com.qmth.exam.reserve.util.ResourceUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "考生信息导入相关接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/std")
@Aac(strict = false, auth = true)
public class StudentImportTaskController extends BaseController {

    @Autowired
    private StudentImportTaskService studentImportService;

    @ApiOperation(value = "考生信息导入分页列表")
    @PostMapping(value = "/page")
    public PageResult<StudentImportTaskVO> page(@RequestBody PagerReq req) {
        return studentImportService.page(req);
    }

    @ApiOperation(value = "考生信息模版下载")
    @GetMapping(value = "/import/template")
    public void downloadTemplate(HttpServletResponse response) {
        exportFile("考生信息导入模板.xlsx", ResourceUtil.getStream("templates/studentImport.xlsx"));
    }

    @Aac(strict = false, auth = false)
    @ApiOperation(value = "考生信息导入")
    @PostMapping(value = "/upload")
    public void upload(MultipartFile file) {
        // LoginUser user = this.curLoginUser();
        // if (!Role.ADMIN.equals(user.getRole())) {
        // throw new StatusException("没有权限");
        // }
        LambdaQueryWrapper<StudentImportTaskEntity> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StudentImportTaskEntity::getStatus, ImportStatus.RUNNING.toString());
        StudentImportTaskEntity exists = studentImportService.getOne(wrapper);
        if (exists != null)
            throw new StatusException("正在导入，请稍后再试");
        try {
            studentImportService.importStudent(1l, file.getOriginalFilename(), file.getInputStream());
        } catch (IOException e) {
            throw new StatusException("文件读取出错", e);
        }
    }
}
